<?php
require_once (__FILE__ . "/../" . "include.php");

class ORMapper {
	
	function setup($object) {
		return DatabaseManager::doUpdateQuery(ORMapperQuery::getCreateTableQuery($object));
	}

    function saveOrUpdate($object) {
    		if (isset($object->id) && is_numeric($object->id)) {
    			return ORMapper::update($object);
    		}
    		else {
    			return DatabaseManager::doUpdateQuery(ORMapperQuery::getInsertQuery($object));
    		}
    }
    
    function update($object, $where = null) {
		return DatabaseManager::doUpdateQuery(ORMapperQuery::getUpdateQuery($object, $where));
    }
    
    function insert($object) {
		$connection = DatabaseManager :: getDatabaseConnection();
		if ($connection != null) {
			if (DatabaseManager::doUpdateQueryWithConnection(ORMapperQuery::getInsertQuery($object), $connection) &&
				($id = DatabaseManager::getLastInsertID($connection)) != null) {
				DatabaseManager::closeDatabaseConnection($connection);	
				$object->id = $id;
				return $object;
			}
			else {
				return null;
			}
		}
		else {
			return null;
		}
    }
    
    function delete($object) {
		return DatabaseManager::doUpdateQuery(ORMapperQuery::getDeleteQuery($object));
    }
    
    function select($object,
		$whereQuery,
		$orderColumnName = null,
		$isOrderReverse = false,
		$limit = null,
		$offset = 0) {
    		$resultArray = DatabaseManager::doSelectQuery(ORMapperQuery::getSelectQuery($object,
			$whereQuery,
			$orderColumnName,
			$isOrderReverse,
			$limit,
			$offset));
		return ORMapper::convertClassType($resultArray, $object);
    }
    
    function convertClassType($resultArray, $object) {
		$className = get_class($object);
		$convertResult = array();
    		for ($i = 0; $i < count($resultArray); $i++) {
    			$nowObject = new $className();
    			foreach ($resultArray[$i] as $key => $value) {
    				$nowObject->$key = $value;
    			}
    			$convertResult[] = $nowObject;
    		}
    		return $convertResult;
    }
    
    function selectOne($object,
		$whereQuery,
		$orderColumnName = null,
		$isOrderReverse = false,
		$limit = null,
		$offset = 0) {
		$result = ORMapper::select($object,
			$whereQuery,
			$orderColumnName = null,
			$isOrderReverse = false,
			$limit = null,
			$offset = 0)	;
		switch (count($result)) {
			case 1:
				return $result[0];
			case 0:
				return null;
			default:
				trigger_error(ORMapperErrorMessages::faultResultCount(), E_USER_ERROR);
				return null;
		}
	}
	
	function validate($object) {
		$objectVars = get_object_vars($object);
		foreach ($objectVars as $varName => $varValue) {
			if (FieldLibrary::isRequiredField($varName) &&
				!isset($varValue)) {
				return false;
			}
		}
		return true;
	}
    
}

?>